sampler ColorSampler : register(s0);

float4 LightColor;
float3 LightPosition;
float Brightness;
float Radius;
bool AttenuateLight;

float ScreenWidth;
float ScreenHeight;

Texture NormalMap;
sampler NormalMapSampler = sampler_state {
	texture = <NormalMap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = mirror;
	AddressV = mirror;
};

Texture DepthMap;
sampler DepthMapSampler = sampler_state {
	texture = <DepthMap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = mirror;
	AddressV = mirror;
};

float4 DeferredLightingPS(float4 color : COLOR0, 
                                 float2 texCoord : TEXCOORD0) : COLOR0
{
	// get base texture and normals
	float4 base = tex2D(ColorSampler, texCoord);	
	float3 normal = tex2D(NormalMapSampler, texCoord).rgb;	
	normal = normal*2.0f-1.0f;
    normal = normalize(normal);
	float depth = tex2D(DepthMapSampler, texCoord);
	
	base.a = 1;
	
	// get pixel position in world coords, we well as light direction and distance
	float3 pixelPosition;
	
	pixelPosition.x = ScreenWidth * texCoord.x;
	pixelPosition.y = ScreenHeight * texCoord.y;
	pixelPosition.z = depth;


	float4 finalShading;
	
	if (depth > 0)
	{
		float3 direction = LightPosition - pixelPosition;
		float distance = 1 / length(LightPosition - pixelPosition) * Brightness;

		// calculate light amount
		float amount = max(dot(normal, normalize(distance)), 0);
		LightColor *= Brightness;
			
		if (AttenuateLight == true)
		{
			float coneAttenuation = saturate(1.0f - length(direction) / Radius);
			finalShading = (distance * amount * coneAttenuation * LightColor);
		}
		else
		{
			finalShading = (distance * amount * LightColor);
		}
	}
	else
	{
		finalShading = float4(0,0,0,0);
	}
	
	finalShading.a = 1;
	
	return finalShading;
}

technique Deferred2DNormalMapping
{
    pass Pass0
    {
        PixelShader = compile ps_2_0 DeferredLightingPS();
    }
}
